// cnstbl.cpp

#include <windows.h>
#include "msc.h"
#include "res.h"
#include "winmain.h"

#include "cnstbl.h"
#include "cns2uni.h"
#include "uni2cns.h"



int WINAPI DllMain ( HINSTANCE hInst, unsigned long dwReason, void *pReserved ) {

   if ( dwReason == DLL_PROCESS_ATTACH ) {
      // \[X̃CX^X
      SetResourceInstance ( hInst ) ;
   }

   return TRUE ;
}



// CNS11643  EUC GR[fBOŎw
// Unicode ̃XJ[lԂ
// ϊłȂ 0 Ԃ
int WINAPI Cns11643ToUnicode ( int C ) {

   int nPlane = 0 ;

   if ( C & 0xFFFF0000 ) {
      int cTmp = ( (unsigned) C ) >> 16 ;
      if ( cTmp < 0x8EA1 || cTmp > 0x8EAF ) return 0 ;
      nPlane = ( cTmp - 0x8EA1 + 1 ) ;
      C &= 0xFFFF ;
   }
   else if ( C & 0xFF00 ) {
      nPlane = 1 ;
   }


   int cHigh = C >> 8 ;
   int cLow = C & 0xFF ;

   const unsigned short *pTable ;

   switch ( nPlane ) {
      case 0 :  if ( ! cHigh && cLow < 0x80 ) return cLow ;
                else                          return 0 ;
      case 1 :  pTable = Cns2uniP1Table ( cHigh ) ;  break ;
      case 2 :  pTable = Cns2uniP2Table ( cHigh ) ;  break ;
      case 3 :  pTable = Cns2uniP3Table ( cHigh ) ;  break ;
      case 4 :  pTable = Cns2uniP4Table ( cHigh ) ;  break ;
      case 5 :  pTable = Cns2uniP5Table ( cHigh ) ;  break ;
      case 6 :  pTable = Cns2uniP6Table ( cHigh ) ;  break ;
      case 7 :  pTable = Cns2uniP7Table ( cHigh ) ;  break ;
      case 8 :  pTable = Cns2uniP8Table ( cHigh ) ;  break ;
      case 9 :  pTable = Cns2uniP9Table ( cHigh ) ;  break ;
      case 10 : pTable = Cns2uniP10Table ( cHigh ) ; break ;
      case 11 : pTable = Cns2uniP11Table ( cHigh ) ; break ;
      case 12 : pTable = Cns2uniP12Table ( cHigh ) ; break ;
      case 13 : pTable = Cns2uniP13Table ( cHigh ) ; break ;
      case 14 : pTable = Cns2uniP14Table ( cHigh ) ; break ;
      case 15 : pTable = Cns2uniP15Table ( cHigh ) ; break ;
      default : return 0 ;
   }

   if ( pTable && cLow >= 0xA0 ) {
      int cUnicode = pTable [ cLow - 0xA0 ] ;
      if ( GetShortArrayBit ( pTable + 0x100 - 0xA0, cLow - 0xA0 ) ) cUnicode += 0x20000 ;
      return cUnicode ;
   }

   return 0 ;
}



// Unicode ̃XJ[lŎw
// CNS11643  EUC GR[fBOŕԂ
// ϊłȂ 0 Ԃ
int WINAPI UnicodeToCns11643 ( int cUnicode ) {

   const unsigned short *pTable ;

   switch ( cUnicode >> 16 ) {
      case 0 : pTable = Uni2cnsP0Table ( ( cUnicode & 0xFFFF ) >> 8 ) ; break ;
      case 2 : pTable = Uni2cnsP2Table ( ( cUnicode & 0xFFFF ) >> 8 ) ; break ;
      default : return 0 ;
   }

   if ( pTable ) {

      int C = pTable [ cUnicode & 0xFF ] ;

      int nBit = 0 ;
      if ( C & 0x80 ) nBit |= 1 ;
      if ( C & 0x8000 ) nBit |= 2 ;

      C &= 0x7F7F ;

      for ( int nPlane = nBit ; nPlane < 0x10 ; nPlane += 4 ) {

         int cResult ;
         if ( nPlane == 0 )      cResult = C ;
         else if ( nPlane == 1 ) cResult = C | 0x8080 ;
         else                    cResult = ( ( 0x8EA1 + nPlane - 1 ) << 16 ) + ( C | 0x8080 ) ;

         if ( Cns11643ToUnicode ( cResult ) == cUnicode ) return cResult ;
      }
   }

   return 0 ;
}



int WINAPI GetDllVersion ( void ) {

   FILEVERSION FileVersion ;
   if ( GetFileVersion ( & FileVersion ) ) return 0 ;

   return TranslateVersionInfo ( & FileVersion ) ;
}



