// mbctbl.cpp

#include <windows.h>
#include <ctype.h>
#include "msc.h"
#include "codepage.h"
#include "res.h"
#include "winmain.h"

#include "mbctbl.h"

#include "uni2mbc.h"
#include "mbc2uni.h"
#include "jhojo2uni.h"
#include "uni2jhojo.h"
#include "gaiji.h"
#include "johab.h"



// for HKSCS
static int Cjkb2hkscs ( int cUnicode ) ;
static int IsBig5Pua ( int cBig5 ) ;



// Gg[|Cg
int WINAPI DllMain ( HINSTANCE hInst, unsigned long dwReason, void *pReserved ) {

   if ( dwReason == DLL_PROCESS_ATTACH ) {
      // \[X̃CX^X
      SetResourceInstance ( hInst ) ;
   }

   return TRUE ;
}



// _~[̃e[uZN^
static const unsigned short *DefaultTableSelecter ( int ) { return NULL ; }



// QoCgR[h UNICODE ɕϊ
int WINAPI MultiByteToUnicode ( int C, int nCodePage ) {

   int cResult ;

   if ( (unsigned) C < 0x100 ) {

      if ( isascii ( C ) ) return C ;

      const unsigned short *pTable ;

      switch ( nCodePage ) {
         case CP_JAPANESE :            pTable = Sjis2uniTable_0080_00FF ; break ;
         case CP_KOREAN :              pTable = Ks2uniTable_0080_00FF ; break ;
         case CP_CHINESE_SIMPLIFIED :  pTable = Gb2uniTable_0080_00FF ; break ;
         case CP_CHINESE_TRADITIONAL : pTable = Big52uniTable_0080_00FF ; break ;
         case CP_CHINESE_HKSCS :       pTable = Hkscs2uniTable_0080_00FF ; break ;
         default :                     pTable = NULL ; break ;
      }

      if ( pTable )                            cResult = pTable [ C - 0x80 ] ;
      else if ( nCodePage == CP_KOREAN_JOHAB ) cResult = JohabToUnicode ( C ) ;
      else                                     cResult = 0 ;

   }
   else {

      const unsigned short *( *pTableSelecter ) ( int ) ;

      switch ( nCodePage ) {
         case CP_JAPANESE :            pTableSelecter = Sjis2uniTable ; break ;
         case CP_KOREAN :              pTableSelecter = Ks2uniTable ; break ;
         case CP_CHINESE_SIMPLIFIED :  pTableSelecter = Gb2uniTable ; break ;
         case CP_CHINESE_TRADITIONAL : pTableSelecter = Big52uniTable ; break ;
         case CP_CHINESE_HKSCS :       pTableSelecter = Hkscs2uniTable ; break ;
         default :                     pTableSelecter = DefaultTableSelecter ; break ;
      }

      const unsigned short *Mbc2uniTable = pTableSelecter ( C >> 8 ) ;

      if ( Mbc2uniTable && ( C & 0xFF ) >= 0x40 ) cResult = Mbc2uniTable [ ( C & 0xFF ) - 0x40 ] ;
      else if ( nCodePage == CP_KOREAN_JOHAB )    cResult = JohabToUnicode ( C ) ;
      else                                        cResult = 0 ;

      if ( nCodePage == CP_CHINESE_TRADITIONAL && IsBig5Pua ( C ) ) cResult = 0 ;
      if ( nCodePage == CP_CHINESE_HKSCS && IsBig5Pua ( C ) ) {
         const unsigned short *pBitTable = Hkscs2uniCjk2Bit ( C >> 8 ) ;
         if ( pBitTable && GetShortArrayBit ( pBitTable, ( C & 0xFF ) ) ) cResult |= 0x20000 ;
      }

   }

   if ( ! cResult ) cResult = GaijiToUnicode ( C, nCodePage ) ;

   return cResult ;
}



// UNICODE QoCgR[hɕϊ
int WINAPI UnicodeToMultiByte ( int cUnicode, int nCodePage ) {

   const unsigned short *( *pTableSelecter ) ( int ) ;

   switch ( nCodePage ) {
      case CP_JAPANESE :            pTableSelecter = Uni2sjisTable ; break ;
      case CP_KOREAN :              pTableSelecter = Uni2ksTable ; break ;
      case CP_CHINESE_SIMPLIFIED :  pTableSelecter = Uni2gbTable ; break ;
      case CP_CHINESE_TRADITIONAL : pTableSelecter = Uni2big5Table ; break ;
      case CP_CHINESE_HKSCS :       pTableSelecter = Uni2hkscsTable ; break ;
      default :                     pTableSelecter = DefaultTableSelecter ; break ;
   }

   const unsigned short *Uni2mbcTable = pTableSelecter ( cUnicode >> 8 ) ;

   int cResult ;
   if ( Uni2mbcTable )                      cResult = Uni2mbcTable [ cUnicode & 0xFF ] ;
   else if ( nCodePage == CP_KOREAN_JOHAB ) cResult = UnicodeToJohab ( cUnicode ) ;
   else                                     cResult = 0 ;

   if ( nCodePage == CP_JAPANESE && cResult < 0x8000 && cResult > 0xFF ) cResult = 0 ;
   if ( nCodePage == CP_CHINESE_TRADITIONAL && IsBig5Pua ( cResult ) ) cResult = 0 ;
   if ( nCodePage == CP_CHINESE_HKSCS && cUnicode > 0xFFFF ) cResult = Cjkb2hkscs ( cUnicode ) ;

   if ( ! cResult ) cResult = UnicodeToGaiji ( cUnicode, nCodePage ) ;

   return cResult ;
}



// CJK Ex.B  HKSCS ɕϊ
static int Cjkb2hkscs ( int cUnicode ) {

   const unsigned short *pBitTable = Cjkb2hkscsBit ( cUnicode >> 12 ) ;
   if ( pBitTable && ! GetShortArrayBit ( pBitTable, cUnicode & 0xFFF ) ) return 0 ;

   for ( int C = 0x80 ; C <= 0xFF ; C ++ ) {
      const unsigned short *pBitTable = Hkscs2uniCjk2Bit ( C ) ;
      if ( ! pBitTable ) continue ;
      const unsigned short *pHkscs2uniTable = Hkscs2uniTable ( C ) ;
      if ( ! pHkscs2uniTable ) continue ;
      for ( int CC = 0 ; CC <= 0xFF/16 ; CC ++ ) {
         int cLow = CC * 16 ;
         for ( int Mask = pBitTable [ CC ] ; Mask ; Mask >>= 1, cLow ++ ) {
            if ( ( Mask & 1 ) && pHkscs2uniTable [ cLow - 0x40 ] == ( cUnicode & 0xFFFF ) ) return ( C << 8 ) + cLow ;
         }
      }
   }

   return 0 ;  // usually never reached
}



// BIG5 ̊ÖȂ 0 ȊOԂ
static int IsBig5Pua ( int cBig5 ) {

   if ( (unsigned) cBig5 <= 0xFF ) return 0 ;

   int cHigh = cBig5 >> 8 ;
   if ( cHigh <= 0xA0 ) return 1 ;
   if ( cHigh >= 0xFA ) return 1 ;

   if ( cBig5 >= 0xC6A1 && cBig5 <= 0xC8FE ) return 1 ;

   return 0 ;
}



// JIS⏕ UNICODE ɕϊ
// ŏʃrbg ON ł OFF ł
int WINAPI JisHojoToUnicode ( int C ) {

   C &= 0x7F7F ;

   const unsigned short *Mbc2uniTable = Jhojo2uniTable ( C >> 8 ) ;
   if ( Mbc2uniTable ) {
      if ( ( C & 0xFF ) < 0x20 ) return 0 ;
      return Mbc2uniTable [ ( C & 0xFF ) - 0x20 ] ;
   }

   return 0 ;
}



// UNICODE JIS⏕ɕϊ
// ŏʃrbg ON ɂĕԂ
int WINAPI UnicodeToJisHojo ( int cUnicode ) {

   const unsigned short *Uni2mbcTable = Uni2sjisTable ( cUnicode >> 8 ) ;
   if ( Uni2mbcTable ) {
      int C = Uni2mbcTable [ cUnicode & 0xFF ] ;
      if ( C >= 0x8000 || C <= 0xFF ) {
         if ( cUnicode <= 0xFF ) C = Uni2jhojoTable_00 [ cUnicode & 0xFF ] ;
         else {
            const unsigned short *pTable = Sjis2jhojoTable ( C >> 8 ) ;
            if ( pTable ) C = pTable [ ( C & 0xFF ) - 0x40 ] ;
            else          C = 0 ;
         }
      }
      if ( C ) return C | 0x8080 ;
   }

   return 0 ;
}



#include "sjis2004.h"



// Shift-JIS 2004  UNICODE ɕϊ
// 2004 łȂ Shift-JIS ̕ϊʂ cUnicode Ɏw肷
// P Unicode ɑȂApszUnicode  NULL i[AϊʂԂ
// Q Unicode ɑȂApszUnicode ɕi[A0 Ԃ
int WINAPI ShiftJis2004ToUnicode ( int cSjis, int cUnicode, const wchar_t **pszUnicode ) {

   *pszUnicode = NULL ;

   if ( cSjis > 0 && cSjis < 0x100 ) return Sjis2uni_2004_SingleByte ( cSjis, cUnicode ) ;

   const unsigned short *pTable = Sjis2uniTable_2004 ( cSjis >> 8 ) ;
   if ( pTable && ( cSjis & 0xFF ) >= 0x40 ) {
      int cResult = pTable [ ( cSjis & 0xFF ) - 0x40 ] ;
      if ( cResult == 0xFFFF ) {
         *pszUnicode = Sjis2uni_2004_Composition ( cSjis ) ;
         return 0 ;
      }
      if ( cResult ) {
         const unsigned short *pBitTable = Sjis2uniCjk2Bit_2004 ( cSjis >> 8 ) ;
         if ( pBitTable && GetShortArrayBit ( pBitTable, cSjis & 0xFF ) ) return cResult + 0x20000 ;
         return cResult ;
      }
   }

   if ( cSjis >= 0x8500 && cSjis <= 0x889E ) cUnicode = 0 ;
   if ( cSjis >= 0xEB00 && cSjis <= 0xFCFF ) cUnicode = 0 ;

   return cUnicode ;
}



// UNICODE  Shift-JIS 2004 ɕϊ
// 2004 łȂ Shift-JIS ̕ϊʂ cSjis Ɏw肷
// szUnicode  NULL ȊOȂ cUnicode  cSjis 𖳎
//   szUnicode PȂAPڂv 0 ȊOAvȂ 0 Ԃ
//   szUnicode QȂAϊʂԂ
int WINAPI UnicodeToShiftJis2004 ( int cUnicode, int cSjis, const wchar_t *szUnicode ) {

   if ( szUnicode ) return Uni2sjis_2004_Composition ( szUnicode ) ;

   if ( cSjis > 0 && cSjis < 0x100 ) {
      if ( cUnicode == Sjis2uni_2004_SingleByte ( cSjis, cUnicode ) ) return cSjis ;
      cSjis = 0 ;
   }

   const unsigned short *pBitTable = Uni2sjisBit_2004 ( cUnicode >> 12 ) ;
   if ( pBitTable && GetShortArrayBit ( pBitTable, cUnicode & 0xFFF ) ) {
      for ( int C = 0xFF ; C >= 0x80 ; C -- ) {
         const unsigned short *pTable = Sjis2uniTable_2004 ( C ) ;
         if ( ! pTable ) continue ;
         for ( int CC = 0x40 ; CC < 0x100 ; CC ++ ) {
            int cResult = pTable [ CC - 0x40 ] ;
            if ( ( cUnicode & 0xFFFF ) != cResult ) continue ;
            const unsigned short *pBitTable = Sjis2uniCjk2Bit_2004 ( C ) ;
            if ( pBitTable && GetShortArrayBit ( pBitTable, CC ) ) cResult += 0x20000 ;
            if ( cUnicode == cResult ) return ( C << 8 ) + CC ;
         }
      }
      return 0 ;
   }

   if ( cSjis ) {
      if      ( cSjis >= 0x8500 && cSjis <= 0x889E ) cSjis = 0 ;
      else if ( cSjis >= 0xEB00 && cSjis <= 0xFCFF ) cSjis = 0 ;
   }

   if ( cSjis ) {
      const unsigned short *pTable = Sjis2uniTable_2004 ( cSjis >> 8 ) ;
      if ( pTable && ( ( cSjis & 0xFF ) < 0x40 || pTable [ ( cSjis & 0xFF ) - 0x40 ] ) ) cSjis = 0 ;
   }

   return cSjis ;
}



int WINAPI IsMbcValidCodePage ( int nCodePage ) {
   return IsMbcDllCodePage ( nCodePage ) ;
}



int WINAPI GetDllVersion ( void ) {

   FILEVERSION FileVersion ;
   if ( GetFileVersion ( & FileVersion ) ) return 0 ;

   return TranslateVersionInfo ( & FileVersion ) ;
}



