// stringex.h

#ifndef STRINGEX_H
#define STRINGEX_H


#ifdef __cplusplus
extern "C" {
#endif


// strend
// wcsend
wchar32_t *wcsend32 ( const wchar32_t *string ) ;

// strcpy
// wcscpy
wchar32_t *wcscpy32 ( wchar32_t *dst, const wchar32_t *src ) ;

// strcat
// wcscat
wchar32_t *wcscat32 ( wchar32_t *dst, const wchar32_t *src ) ;


// strlcpylen
// wcslcpylen
size_t wcslcpylen16 ( wchar16_t *dst, const wchar16_t *src, size_t srclen, size_t bufsiz ) ;
size_t wcslcpylen32 ( wchar32_t *dst, const wchar32_t *src, size_t srclen, size_t bufsiz ) ;
size_t mbslcpylen ( char *dst, const char *src, size_t srclen, size_t bufsiz ) ;

// strlcpy
// wcslcpy
#define wcslcpy16(dst,src,bufsiz)   wcslcpylen16 ( dst, src, (size_t) -1, bufsiz )
#define wcslcpy32(dst,src,bufsiz)   wcslcpylen32 ( dst, src, (size_t) -1, bufsiz )
#define mbslcpy(dst,src,bufsiz)     mbslcpylen ( dst, src, (size_t) -1, bufsiz )


size_t strlcat ( char *dst, const char *src, size_t bufsiz ) ;
size_t wcslcat ( wchar_t *dst, const wchar_t *src, size_t bufsiz ) ;
size_t wcslcat16 ( wchar16_t *dst, const wchar16_t *src, size_t bufsiz ) ;
size_t wcslcat32 ( wchar32_t *dst, const wchar32_t *src, size_t bufsiz ) ;
size_t mbslcat ( char *dst, const char *src, size_t bufsiz ) ;


char *strmove ( char *dst, const char *src ) ;
wchar_t *wcsmove ( wchar_t *dst, const wchar_t *src ) ;
wchar32_t *wcsmove32 ( wchar32_t *dst, const wchar32_t *src ) ;
#define mbsmove   strmove


char *strinsert ( char *dst, const char *src ) ;
wchar_t *wcsinsert ( wchar_t *dst, const wchar_t *src ) ;
wchar32_t *wcsinsert32 ( wchar32_t *dst, const wchar32_t *src ) ;
#define mbsinsert    strinsert


size_t strreplace ( char *string, const char *pattern, const char *replace, char **pnext, size_t max ) ;
size_t wcsreplace ( wchar_t *string, const wchar_t *pattern, const wchar_t *replace, wchar_t **pnext, size_t max ) ;
size_t wcsreplace32 ( wchar32_t *string, const wchar32_t *pattern, const wchar32_t *replace, wchar32_t **pnext, size_t max ) ;
size_t mbsreplace ( char *string, const char *pattern, const char *replace, char **pnext, size_t max ) ;


char *strlset ( char *dst, int c, size_t bufsiz ) ;
wchar_t *wcslset ( wchar_t *dst, int c, size_t bufsiz ) ;
wchar32_t *wcslset32 ( wchar32_t *dst, int c, size_t bufsiz ) ;
char *mbslset ( char *dst, int c, size_t bufsiz ) ;


int strncmp_i ( const char *string1, const char *string2, size_t count, int isignore ) ;
int wcsncmp_i ( const wchar_t *string1, const wchar_t *string2, size_t count, int isignore ) ;
int wcsncmp32_i ( const wchar32_t *string1, const wchar32_t *string2, size_t count, int isignore ) ;
int mbsncmp_i ( const char *string1, const char *string2, size_t count, int isignore ) ;

// #define strcmp(s1,s2)      strncmp_i   ( s1, s2, (size_t) -1, 0 )
// #define wcscmp(s1,s2)      wcsncmp_i   ( s1, s2, (size_t) -1, 0 )
#define wcscmp32(s1,s2)       wcsncmp32_i ( s1, s2, (size_t) -1, 0 )
#define mbscmp    strcmp

#define strncmp(s1,s2,n)      strncmp_i   ( s1, s2, n, 0 )
#define wcsncmp(s1,s2,n)      wcsncmp_i   ( s1, s2, n, 0 )
#define wcsncmp32(s1,s2,n)    wcsncmp32_i ( s1, s2, n, 0 )
#define mbsncmp   strncmp

#define stricmp(s1,s2)        strncmp_i   ( s1, s2, (size_t) -1, 1 )
#define wcsicmp(s1,s2)        wcsncmp_i   ( s1, s2, (size_t) -1, 1 )
#define wcsicmp32(s1,s2)      wcsncmp32_i ( s1, s2, (size_t) -1, 1 )
#define mbsicmp(s1,s2)        mbsncmp_i   ( s1, s2, (size_t) -1, 1 )

#define strnicmp(s1,s2,n)     strncmp_i   ( s1, s2, n, 1 )
#define wcsnicmp(s1,s2,n)     wcsncmp_i   ( s1, s2, n, 1 )
#define wcsnicmp32(s1,s2,n)   wcsncmp32_i ( s1, s2, n, 1 )
#define mbsnicmp(s1,s2,n)     mbsncmp_i   ( s1, s2, n, 1 )


int strrncmp_i ( const char *string1, const char *string2, size_t count, int isignore ) ;
int wcsrncmp_i ( const wchar_t *string1, const wchar_t *string2, size_t count, int isignore ) ;
int wcsrncmp32_i ( const wchar32_t *string1, const wchar32_t *string2, size_t count, int isignore ) ;
int mbsrncmp_i ( const char *string1, const char *string2, size_t count, int isignore ) ;

#define strrcmp(s1,s2)        strrncmp_i   ( s1, s2, (size_t) -1, 0 )
#define wcsrcmp(s1,s2)        wcsrncmp_i   ( s1, s2, (size_t) -1, 0 )
#define wcsrcmp32(s1,s2)      wcsrncmp32_i ( s1, s2, (size_t) -1, 0 )
#define mbsrcmp(s1,s2)        mbsrncmp_i   ( s1, s2, (size_t) -1, 0 )

#define strrncmp(s1,s2,n)     strrncmp_i   ( s1, s2, n, 0 )
#define wcsrncmp(s1,s2,n)     wcsrncmp_i   ( s1, s2, n, 0 )
#define wcsrncmp32(s1,s2,n)   wcsrncmp32_i ( s1, s2, n, 0 )
#define mbsrncmp(s1,s2,n)     mbsrncmp_i   ( s1, s2, n, 0 )

#define strricmp(s1,s2)       strrncmp_i   ( s1, s2, (size_t) -1, 1 )
#define wcsricmp(s1,s2)       wcsrncmp_i   ( s1, s2, (size_t) -1, 1 )
#define wcsricmp32(s1,s2)     wcsrncmp32_i ( s1, s2, (size_t) -1, 1 )
#define mbsricmp(s1,s2)       mbsrncmp_i   ( s1, s2, (size_t) -1, 1 )

#define strrnicmp(s1,s2,n)    strrncmp_i   ( s1, s2, n, 1 )
#define wcsrnicmp(s1,s2,n)    wcsrncmp_i   ( s1, s2, n, 1 )
#define wcsrnicmp32(s1,s2,n)  wcsrncmp32_i ( s1, s2, n, 1 )
#define mbsrnicmp(s1,s2,n)    mbsrncmp_i   ( s1, s2, n, 1 )


int strheadcmp_i ( const char *string1, const char *string2, int isignore ) ;
int wcsheadcmp_i ( const wchar_t *string1, const wchar_t *string2, int isignore ) ;
int wcsheadcmp32_i ( const wchar32_t *string1, const wchar32_t *string2, int isignore ) ;
int mbsheadcmp_i ( const char *string1, const char *string2, int isignore ) ;

#define strheadcmp(s1,s2)     strheadcmp_i   ( s1, s2, 0 )
#define wcsheadcmp(s1,s2)     wcsheadcmp_i   ( s1, s2, 0 )
#define wcsheadcmp32(s1,s2)   wcsheadcmp32_i ( s1, s2, 0 )
#define mbsheadcmp   strheadcmp

#define strheadicmp(s1,s2)    strheadcmp_i   ( s1, s2, 1 )
#define wcsheadicmp(s1,s2)    wcsheadcmp_i   ( s1, s2, 1 )
#define wcsheadicmp32(s1,s2)  wcsheadcmp32_i ( s1, s2, 1 )
#define mbsheadicmp(s1,s2)    mbsheadcmp_i   ( s1, s2, 1 )


int strtailcmp_i ( const char *string1, const char *string2, int isignore ) ;
int wcstailcmp_i ( const wchar_t *string1, const wchar_t *string2, int isignore ) ;
int wcstailcmp32_i ( const wchar32_t *string1, const wchar32_t *string2, int isignore ) ;
int mbstailcmp_i ( const char *string1, const char *string2, int isignore ) ;

#define strtailcmp(s1,s2)     strtailcmp_i   ( s1, s2, 0 )
#define wcstailcmp(s1,s2)     wcstailcmp_i   ( s1, s2, 0 )
#define wcstailcmp32(s1,s2)   wcstailcmp32_i ( s1, s2, 0 )
#define mbstailcmp(s1,s2)     mbstailcmp_i   ( s1, s2, 0 )

#define strtailicmp(s1,s2)    strtailcmp_i   ( s1, s2, 1 )
#define wcstailicmp(s1,s2)    wcstailcmp_i   ( s1, s2, 1 )
#define wcstailicmp32(s1,s2)  wcstailcmp32_i ( s1, s2, 1 )
#define mbstailicmp(s1,s2)    mbstailcmp_i   ( s1, s2, 1 )


char *strstr_i_r ( const char *string, const char *pattern, int isignore, int islast ) ;
wchar_t *wcsstr_i_r ( const wchar_t *string, const wchar_t *pattern, int isignore, int islast ) ;
wchar32_t *wcsstr32_i_r ( const wchar32_t *string, const wchar32_t *pattern, int isignore, int islast ) ;
char *mbsstr_i_r ( const char *string, const char *pattern, int isignore, int islast ) ;

// #define strstr(s,p)     strstr_i_r   ( s, p, 0, 0 )
// #define wcsstr(s,p)     wcsstr_i_r   ( s, p, 0, 0 )
#define wcsstr32(s,p)      wcsstr32_i_r ( s, p, 0, 0 )
#define mbsstr(s,p)        mbsstr_i_r   ( s, p, 0, 0 )

#define strstri(s,p)       strstr_i_r   ( s, p, 1, 0 )
#define wcsstri(s,p)       wcsstr_i_r   ( s, p, 1, 0 )
#define wcsstri32(s,p)     wcsstr32_i_r ( s, p, 1, 0 )
#define mbsstri(s,p)       mbsstr_i_r   ( s, p, 1, 0 )

#define strrstr(s,p)       strstr_i_r   ( s, p, 0, 1 )
#define wcsrstr(s,p)       wcsstr_i_r   ( s, p, 0, 1 )
#define wcsrstr32(s,p)     wcsstr32_i_r ( s, p, 0, 1 )
#define mbsrstr(s,p)       mbsstr_i_r   ( s, p, 0, 1 )

#define strrstri(s,p)      strstr_i_r   ( s, p, 1, 1 )
#define wcsrstri(s,p)      wcsstr_i_r   ( s, p, 1, 1 )
#define wcsrstri32(s,p)    wcsstr32_i_r ( s, p, 1, 1 )
#define mbsrstri(s,p)      mbsstr_i_r   ( s, p, 1, 1 )


char *strchr_i_r ( const char *string, int c, int isignore, int islast ) ;
wchar_t *wcschr_i_r ( const wchar_t *string, int c, int isignore, int islast ) ;
wchar16_t *wcschr16_i_r ( const wchar16_t *string, int c, int isignore, int islast ) ;
wchar32_t *wcschr32_i_r ( const wchar32_t *string, int c, int isignore, int islast ) ;
char *mbschr_i_r ( const char *string, int c, int isignore, int islast ) ;

// #define strchr(s,c)     strchr_i_r   ( s, c, 0, 0 )
// #define wcschr(s,c)     wcschr_i_r   ( s, c, 0, 0 )
#define wcschr16(s,c)      wcschr16_i_r ( s, c, 0, 0 )
#define wcschr32(s,c)      wcschr32_i_r ( s, c, 0, 0 )
#define mbschr(s,c)        mbschr_i_r   ( s, c, 0, 0 )

#define strchri(s,c)       strchr_i_r   ( s, c, 1, 0 )
#define wcschri(s,c)       wcschr_i_r   ( s, c, 1, 0 )
#define wcschri16(s,c)     wcschr16_i_r ( s, c, 1, 0 )
#define wcschri32(s,c)     wcschr32_i_r ( s, c, 1, 0 )
#define mbschri(s,c)       mbschr_i_r   ( s, c, 1, 0 )

// #define strrchr(s,c)    strchr_i_r   ( s, c, 0, 1 )
// #define wcsrchr(s,c)    wcschr_i_r   ( s, c, 0, 1 )
#define wcsrchr16(s,c)     wcschr16_i_r ( s, c, 0, 1 )
#define wcsrchr32(s,c)     wcschr32_i_r ( s, c, 0, 1 )
#define mbsrchr(s,c)       mbschr_i_r   ( s, c, 0, 1 )

#define strrchri(s,c)      strchr_i_r   ( s, c, 1, 1 )
#define wcsrchri(s,c)      wcschr_i_r   ( s, c, 1, 1 )
#define wcsrchri16(s,c)    wcschr16_i_r ( s, c, 1, 1 )
#define wcsrchri32(s,c)    wcschr32_i_r ( s, c, 1, 1 )
#define mbsrchri(s,c)      mbschr_i_r   ( s, c, 1, 1 )


// strspn
// wcsspn
size_t wcsspn32 ( const wchar32_t *string, const wchar32_t *keys ) ;
size_t mbsspn ( const char *string, const char *keys ) ;

// strcspn
// wcscspn
size_t wcscspn32 ( const wchar32_t *string, const wchar32_t *keys ) ;
size_t mbscspn ( const char *string, const char *keys ) ;

char *strspnp ( const char *string, const char *keys ) ;
wchar_t *wcsspnp ( const wchar_t *string, const wchar_t *keys ) ;
wchar32_t *wcsspnp32 ( const wchar32_t *string, const wchar32_t *keys ) ;
char *mbsspnp ( const char *string, const char *keys ) ;

// strpbrk
// wcspbrk
wchar32_t *wcspbrk32 ( const wchar32_t *string, const wchar32_t *keys ) ;
char *mbspbrk ( const char *string, const char *keys ) ;


char *strlwr ( char *string ) ;
wchar_t *wcslwr ( wchar_t *string ) ;
wchar32_t *wcslwr32 ( wchar32_t *string ) ;
char *mbslwr ( char *string ) ;
wchar_t *wcslwrex ( wchar_t *string ) ;

char *strupr ( char *string ) ;
wchar_t *wcsupr ( wchar_t *string ) ;
wchar32_t *wcsupr32 ( wchar32_t *string ) ;
char *mbsupr ( char *string ) ;
wchar_t *wcsuprex ( wchar_t *string ) ;


// strrev
// wcsrev
wchar16_t *wcsrev16 ( wchar16_t *string ) ;
wchar32_t *wcsrev32 ( wchar32_t *string ) ;
char *mbsrev ( char *string ) ;


char *strtrunc ( char *string, size_t len ) ;
wchar_t *wcstrunc ( wchar_t *string, size_t len ) ;
wchar16_t *wcstrunc16 ( wchar16_t *string, size_t len ) ;
wchar32_t *wcstrunc32 ( wchar32_t *string, size_t len ) ;
char *mbstrunc ( char *string, size_t len ) ;


// strlen
// wcslen
size_t wcslen32 ( const wchar32_t *string ) ;
size_t mbslen ( const char *string ) ;


// strduplen_max_expand
// wcsduplen_max_expand
wchar32_t *wcsduplen_max_expand32 ( const wchar32_t *string, size_t srclen, size_t maxlen, size_t addlen ) ;
#define wcsduplen32(string,srclen)              wcsduplen_max_expand32 ( string, srclen, (size_t) -1, 0 )
#define wcsdup_expand32(string,maxlen,addlen)   wcsduplen_max_expand32 ( string, (size_t) -1, maxlen, addlen )
#define wcsdup_max32(string,maxlen)             wcsduplen_max_expand32 ( string, (size_t) -1, maxlen, 0 )
#define wcsdup32(string)                        wcsduplen_max_expand32 ( string, (size_t) -1, (size_t) -1, 0 )


char *strldup ( const char *string, size_t bufsiz ) ;
wchar_t *wcsldup ( const wchar_t *string, size_t bufsiz ) ;
wchar16_t *wcsldup16 ( const wchar16_t *string, size_t bufsiz ) ;
wchar32_t *wcsldup32 ( const wchar32_t *string, size_t bufsiz ) ;
char *mbsldup ( const char *string, size_t bufsiz ) ;


int mbsbtype ( const char *string, size_t count ) ;
int ismbslead ( const char *string, const char *current ) ;
int ismbstrail ( const char *string, const char *current ) ;


char *strtokex ( char *string, char **pnext ) ;
wchar_t *wcstokex ( wchar_t *string, wchar_t **pnext ) ;
wchar32_t *wcstokex32 ( wchar32_t *string, wchar32_t **pnext ) ;


void *memchrex_i ( const void *string, int64_t c, size_t count, size_t charsize, int isignore ) ;
#define memchrex(s,c,n,z)     memchrex_i ( s, c, n, z, 0 )
#define memchriex(s,c,n,z)    memchrex_i ( s, c, n, z, 1 )
// #define memchr(s,c,n)      memchrex_i ( s, (unsigned) c, ( n ) / sizeof(char), sizeof(char), 0 )

void *memrchrex_i ( const void *string, int64_t c, size_t count, size_t charsize, int isignore ) ;
#define memrchrex(s,c,n,z)    memrchrex_i ( s, c, n, z, 0 )
#define memrchriex(s,c,n,z)   memrchrex_i ( s, c, n, z, 1 )
#define memrchr(s,c,n)        memrchrex_i ( s, (unsigned) c, ( n ) / sizeof(char), sizeof(char), 0 )


int memcmpex_i ( const void *string1, const void *string2, size_t count, size_t charsize, int isignore ) ;
#define memcmpex(s1,s2,n,z)      memcmpex_i ( s1, s2, n, z, 0 )
#define memicmpex(s1,s2,n,z)     memcmpex_i ( s1, s2, n, z, 1 )
#define memicmp(s1,s2,n)         memcmpex_i ( s1, s2, ( n ) / sizeof(char), sizeof(char), 1 )


void *memsetex ( void *buffer, int64_t c, size_t count, size_t charsize ) ;
#define memzeroex(p,n,z)      memsetex ( p, 0, n, z )


void *memdup ( void *src, size_t len ) ;
void *memdup_expand ( void *src, size_t len, size_t addlen ) ;



#define wcsend16           wcsend
#define wcscpy16           wcscpy
#define wcscat16           wcscat
// #define wcslcpy16       wcslcpy
// #define wcslcat16       wcslcat
#define wcsmove16          wcsmove
#define wcsinsert16        wcsinsert
#define wcsreplace16       wcsreplace
#define wcslset16          wcslset
#define wcscmp16           wcscmp
#define wcsncmp16          wcsncmp
#define wcsicmp16          wcsicmp
#define wcsnicmp16         wcsnicmp
#define wcsrcmp16          wcsrcmp
#define wcsricmp16         wcsricmp
#define wcsrncmp16         wcsrncmp
#define wcsrnicmp16        wcsrnicmp
#define wcsheadcmp16       wcsheadcmp
#define wcsheadicmp16      wcsheadicmp
#define wcstailcmp16       wcstailcmp
#define wcstailicmp16      wcstailicmp
#define wcsstr16           wcsstr
#define wcsstri16          wcsstri
#define wcsrstr16          wcsrstr
#define wcsrstri16         wcsrstri
// #define wcschr16        wcschr
// #define wcschri16       wcschri
// #define wcsrchr16       wcsrchr
// #define wcsrchri16      wcsrchri
// #define wcsrev16        wcsrev
// #define wcstrunc16      wcstrunc
#define wcslen16           wcslen
#define wcsduplen_expand16 wcsduplen_expand
#define wcsdup_expand16    wcsdup_expand
#define wcsdup16           wcsdup
#define wcsdup_max16       wcsdup_max
#define wcstokex16         wcstokex


#ifdef __cplusplus
}
#endif



#if defined DEBUG_CONST_STRING && defined __cplusplus    // for debug

#define FUNC_STR(func,type)   \
   static inline const type *func##_ ( const type *s ) { return func ( s ) ; } \
   static inline       type *func##_ (       type *s ) { return func ( s ) ; }

#define FUNC_STR_STR(func,type)  \
   static inline const type *func##_ ( const type *s1, const type *s2 ) { return func ( s1, s2 ) ; } \
   static inline       type *func##_ (       type *s1, const type *s2 ) { return func ( s1, s2 ) ; }

#define FUNC_STR_STR_INT_INT(func,type)   \
   static inline const type *func##_ ( const type *s1, const type *s2, int n1, int n2 ) { return func ( s1, s2, n1, n2 ) ; } \
   static inline       type *func##_ (       type *s1, const type *s2, int n1, int n2 ) { return func ( s1, s2, n1, n2 ) ; }

#define FUNC_STR_INT_INT_INT(func,type)   \
   static inline const type *func##_ ( const type *s, int c, int n1, int n2 ) { return func ( s, c, n1, n2 ) ; } \
   static inline       type *func##_ (       type *s, int c, int n1, int n2 ) { return func ( s, c, n1, n2 ) ; }

#define FUNC_STR_INT_SIZET(func,type)  \
   static inline const type *func##_ ( const type *s, int c, size_t i ) { return func ( s, c, i ) ; } \
   static inline       type *func##_ (       type *s, int c, size_t i ) { return func ( s, c, i ) ; }

#define FUNC_STR_INT64_SIZET_SIZET_INT(func,type)  \
   static inline const type *func##_ ( const type *s, int64_t c, size_t i1, size_t i2, int n ) { return func ( s, c, i1, i2, n ) ; } \
   static inline       type *func##_ (       type *s, int64_t c, size_t i1, size_t i2, int n ) { return func ( s, c, i1, i2, n ) ; }

FUNC_STR ( wcsend32, wchar32_t )

FUNC_STR_STR_INT_INT ( strstr_i_r, char )
FUNC_STR_STR_INT_INT ( wcsstr_i_r, wchar_t )
FUNC_STR_STR_INT_INT ( wcsstr32_i_r, wchar32_t )
FUNC_STR_STR_INT_INT ( mbsstr_i_r, char )

FUNC_STR_INT_INT_INT ( strchr_i_r, char )
FUNC_STR_INT_INT_INT ( wcschr_i_r, wchar_t )
FUNC_STR_INT_INT_INT ( wcschr16_i_r, wchar16_t )
FUNC_STR_INT_INT_INT ( wcschr32_i_r, wchar32_t )
FUNC_STR_INT_INT_INT ( mbschr_i_r, char )

FUNC_STR_STR ( strspnp, char )
FUNC_STR_STR ( wcsspnp, wchar_t )
FUNC_STR_STR ( wcsspnp32, wchar32_t )
FUNC_STR_STR ( mbsspnp, char )

FUNC_STR_STR ( wcspbrk32, wchar32_t )
FUNC_STR_STR ( mbspbrk, char )

FUNC_STR_INT64_SIZET_SIZET_INT ( memchrex_i, void )
FUNC_STR_INT64_SIZET_SIZET_INT ( memrchrex_i, void )

#undef FUNC_STR
#undef FUNC_STR_STR
#undef FUNC_STR_STR_INT_INT
#undef FUNC_STR_INT_INT_INT
#undef FUNC_STR_INT_SIZET
#undef FUNC_STR_INT64_SIZET_SIZET_INT

#define wcsend32     wcsend32_

#define strstr_i_r   strstr_i_r_
#define wcsstr_i_r   wcsstr_i_r_
#define wcsstr32_i_r wcsstr32_i_r_
#define mbsstr_i_r   mbsstr_i_r_

#define strchr_i_r   strchr_i_r_
#define wcschr_i_r   wcschr_i_r_
#define wcschr16_i_r wcschr16_i_r_
#define wcschr32_i_r wcschr32_i_r_
#define mbschr_i_r   mbschr_i_r_

#define strspnp      strspnp_
#define wcsspnp      wcsspnp_
#define mbsspnp      mbsspnp_
#define mbspbrk      mbspbrk_

#define memchrex_i   memchrex_i_
#define memrchrex_i  memrchrex_i_

#endif // defined DEBUG_CONST_STRING && defined __cplusplus



#endif


