// conv_abort.cpp

#include <windows.h>
#include <stdlib.h>
#include <limits.h>
#include "msc.h"
#include "conv_abort.h"


#define ABORT_TABLE_SIZE      8

typedef struct ABORT_DATA_ {
   short AbortFlag [ ABORT_TABLE_SIZE ] ;
   size_t ThreadCount [ ABORT_TABLE_SIZE ] ;
   struct ABORT_DATA_ *pNext ;
} ABORT_DATA ;

static ABORT_DATA *pStart ;



////////////////////////////////////////////
//                                  //
////////////////////////////////////////////



static CRITICAL_SECTION AbortTableSection ;

static size_t GetAvailableIndex ( void ) ;
static void FreeAbortData ( void ) ;



// ~tOe[u
//  0 As 0 ȊOԂ
int AllocAbortTable ( void ) {

   if ( InitializeCriticalSectionEx ( & AbortTableSection ) ) return 1 ;

   pStart = (ABORT_DATA*) malloc ( sizeof(ABORT_DATA) ) ;
   if ( pStart ) memzero ( pStart, sizeof(ABORT_DATA) ) ;
   else          return 1 ;

   return 0 ;
}



// ~tOe[u
void FreeAbortTable ( void ) {

   FreeAbortData () ;

   DeleteCriticalSection ( & AbortTableSection ) ;

   return ;
}



////////////////////////////////////////////
//                  s                  //
////////////////////////////////////////////



static size_t nCurrentIndex ;



// ~tOe[uɓo^
//  0 As 0 ȊOԂ
int RegisterAbortFlag ( volatile const short **pAbortFlagAddress ) {

   int nResult = 1 ;

   EnterCriticalSection ( & AbortTableSection ) ;
   __try {

      if ( nCurrentIndex == SIZE_MAX ) nCurrentIndex = GetAvailableIndex () ;

      size_t nIndex = nCurrentIndex ;

      for ( ABORT_DATA *pTable = pStart, *pNext ; ; pTable = pNext ) {

         if ( nIndex < ABORT_TABLE_SIZE ) {
            *pAbortFlagAddress = & pTable->AbortFlag [ nIndex ] ;
            pTable->ThreadCount [ nIndex ] ++ ;
            nResult = 0 ;
            break ;
         }

         nIndex -= ABORT_TABLE_SIZE ;
         pNext = pTable->pNext ;
         if ( ! pNext ) break ;
      }

   } __finally {
      LeaveCriticalSection ( & AbortTableSection ) ;
   }

   return nResult ;
}



// ~tOe[uo^폜
int UnregisterAbortFlag ( volatile const short *pAbortFlag ) {

   EnterCriticalSection ( & AbortTableSection ) ;
   __try {

      for ( ABORT_DATA *pTable = pStart, *pNext ; ; pTable = pNext ) {

         size_t nIndex = pAbortFlag - pTable->AbortFlag ;
         if ( nIndex < ABORT_TABLE_SIZE ) {
            pTable->ThreadCount [ nIndex ] -- ;
            break ;
         }

         pNext = pTable->pNext ;
         if ( ! pNext ) break ;
      }

   } __finally {
      LeaveCriticalSection ( & AbortTableSection ) ;
   }

   return 0 ;
}



// ~tO ON 
int SetAbortFlag ( void ) {

   EnterCriticalSection ( & AbortTableSection ) ;
   __try {

      size_t nIndex = nCurrentIndex ;

      for ( ABORT_DATA *pTable = pStart, *pNext ; ; pTable = pNext ) {

         if ( nIndex < ABORT_TABLE_SIZE ) {
            if ( pTable->ThreadCount [ nIndex ] ) {
               pTable->AbortFlag [ nIndex ] = TRUE ;
               nCurrentIndex = GetAvailableIndex () ;
            }
            break ;
         }

         nIndex -= ABORT_TABLE_SIZE ;
         pNext = pTable->pNext ;
         if ( ! pNext ) break ;
      }

#if 0
      char buffer [ 0x100 ] ;
      wsprintf ( buffer, "%d", nCurrentIndex ) ;
      MessageBox ( NULL, buffer, "", MB_OK ) ;
#endif

   } __finally {
      LeaveCriticalSection ( & AbortTableSection ) ;
   }

   return 0 ;
}



// ~tOe[uŖgp̃CfbNX擾
// s SIZE_MAX Ԃ
static size_t GetAvailableIndex ( void ) {

   size_t nIndex = 0 ;

   for ( ABORT_DATA *pTable = pStart, *pNext ; ; pTable = pNext ) {

      for ( int nCount = 0 ; nCount < ABORT_TABLE_SIZE ; nCount ++ ) {
         if ( ! pTable->ThreadCount [ nCount ] ) {
            pTable->AbortFlag [ nCount ] = FALSE ;
            return nIndex + nCount ;
         }
      }

      nIndex += ABORT_TABLE_SIZE ;
      pNext = pTable->pNext ;
      if ( ! pNext ) {
         pNext = (ABORT_DATA*) malloc ( sizeof(ABORT_DATA) ) ;
         if ( pNext ) memzero ( pNext, sizeof(ABORT_DATA) ) ;
         else         break ;
         pTable->pNext = pNext ;
      }
   }

   return SIZE_MAX ;
}



// ~tOe[ũ[
static void FreeAbortData ( void ) {

   for ( ABORT_DATA *pTable = pStart, *pNext ; ; pTable = pNext ) {

      pNext = pTable->pNext ;
      free ( pTable ) ;

      if ( ! pNext ) break ;
   }

   return ;
}



