// res.h

#ifndef RES_H
#define RES_H

#ifdef __cplusplus
extern "C" {
#endif


HINSTANCE GetResourceInstance ( void ) ;
HINSTANCE SetResourceInstance ( HINSTANCE hNewInstance ) ;

int GetResourceLanguage ( void ) ;
int SetResourceLanguage ( int nLangId ) ;

HINSTANCE GetSatelliteResourceInstance ( void ) ;
HINSTANCE SetSatelliteResourceInstance ( HINSTANCE hNewInstance ) ;


#define LANGID_NEUTRAL     MAKELANGID ( LANG_NEUTRAL, SUBLANG_NEUTRAL )
#define LANGID_ENGLISH     MAKELANGID ( LANG_ENGLISH, SUBLANG_ENGLISH_US )
#define LANGID_JAPANESE    MAKELANGID ( LANG_JAPANESE, SUBLANG_DEFAULT )


int LoadStringExA ( int nId, char *szBuffer, int nBufferSize ) ;
int LoadStringExW ( int nId, wchar_t *szBuffer, int nBufferSize ) ;
const char *GetStringA ( int nId ) ;
const wchar_t *GetStringW ( int nId ) ;
char *GetStringAllocA ( int nId ) ;
wchar_t *GetStringAllocW ( int nId ) ;


HICON GetIconEx ( int nId, int nX, int nY ) ;
#define GetIcon(n)      GetIconEx ( n, 0, 0 )


typedef struct {
   unsigned short nHeight ;
   unsigned short nWeight ;
   unsigned char IsItalic ;
   unsigned char nCharSet ;
   char szFaceName [ LF_FACESIZE ] ;
} DIALOGFONTA ;

typedef struct {
   unsigned short nHeight ;
   unsigned short nWeight ;
   unsigned char IsItalic ;
   unsigned char nCharSet ;
   wchar_t szFaceName [ LF_FACESIZE ] ;
} DIALOGFONTW ;

int GetDialogFontA ( int nTemplate, DIALOGFONTA *pDialogFont ) ;
int GetDialogFontW ( int nTemplate, DIALOGFONTW *pDialogFont ) ;
int GetDialogFontHeight ( int nTemplate ) ;


INT_PTR DialogBoxParamExA ( int nTemplate, HWND hWnd, DLGPROC pDialogProc, LPARAM lParam ) ;
INT_PTR DialogBoxParamExW ( int nTemplate, HWND hWnd, DLGPROC pDialogProc, LPARAM lParam ) ;
HWND CreateDialogParamExA ( int nTemplate, HWND hParent, DLGPROC pDialogProc, LPARAM lParam ) ;
HWND CreateDialogParamExW ( int nTemplate, HWND hParent, DLGPROC pDialogProc, LPARAM lParam ) ;

#define DialogBoxExA(t,w,p)      DialogBoxParamExA ( t, w, p, 0 )
#define DialogBoxExW(t,w,p)      DialogBoxParamExW ( t, w, p, 0 )
#define CreateDialogExA(t,w,p)   CreateDialogParamExA ( t, w, p, 0 )
#define CreateDialogExW(t,w,p)   CreateDialogParamExW ( t, w, p, 0 )


typedef struct {
   unsigned short wMajor ;
   unsigned short wMinor ;
   unsigned short wBuild ;
   unsigned short wPrivate ;
} FILEVERSION ;


int GetFileVersion ( FILEVERSION *pFileVersion ) ;
int TranslateVersionInfo ( const FILEVERSION *pFileVersion ) ;
int CompareFileVersion ( const FILEVERSION *pFileVersion1, const FILEVERSION *pFileVersion2 ) ;
int EnumVersionInfo ( int ( CALLBACK *Callback ) ( int nLevel, const wchar_t *szKey, const wchar_t *szValue, const long *pValue, size_t ValueLength, LPARAM lParam ), LPARAM lParam ) ;
const char *GetVersionCopyrightA ( void ) ;
const wchar_t *GetVersionCopyrightW ( void ) ;


#ifdef UNICODE
#define LoadStringEx LoadStringExW
#define GetString GetStringW
#define GetStringAlloc GetStringAllocW
#define DIALOGFONT DIALOGFONTW
#define GetDialogFont GetDialogFontW
#define DialogBoxParamEx DialogBoxParamExW
#define CreateDialogParamEx CreateDialogParamExW
#define DialogBoxEx DialogBoxExW
#define CreateDialogEx CreateDialogExW
#define GetVersionCopyright GetVersionCopyrightW
#else
#define LoadStringEx LoadStringExA
#define GetString GetStringA
#define GetStringAlloc GetStringAllocA
#define DIALOGFONT DIALOGFONTA
#define GetDialogFont GetDialogFontA
#define DialogBoxParamEx DialogBoxParamExA
#define CreateDialogParamEx CreateDialogParamExA
#define DialogBoxEx DialogBoxExA
#define CreateDialogEx CreateDialogExA
#define GetVersionCopyright GetVersionCopyrightA
#endif


#ifdef __cplusplus
}
#endif

#endif


